import sys
import requests
import re
import xbmc
import xbmcgui
import xbmcplugin
import xbmcaddon
import json
import base64
from urllib.parse import urlencode, parse_qsl
# Funzioni importate
from login import check_credentials

# Informazioni sull'addon
addon = xbmcaddon.Addon()
addon_handle = int(sys.argv[1])
base_url = sys.argv[0]

# Variabili globali per le impostazioni GitHub
REPO_OWNER = None
REPO_NAME = None
GITHUB_TOKEN = None

def check_github_settings():
    global REPO_OWNER, REPO_NAME, GITHUB_TOKEN
    
    REPO_OWNER = addon.getSetting('repo_owner')
    REPO_NAME = addon.getSetting('repo_name')
    GITHUB_TOKEN = addon.getSetting('github_token')

    missing_settings = []
    if not REPO_OWNER:
        missing_settings.append('Owner del repository')
    if not REPO_NAME:
        missing_settings.append('Nome del repository')
    if not GITHUB_TOKEN:
        missing_settings.append('Token GitHub')

    if missing_settings:
        dialog = xbmcgui.Dialog()
        dialog.ok('Impostazioni mancanti', 'Le seguenti impostazioni sono mancanti:\n' + '\n'.join(missing_settings))
        addon.openSettings()
        return False

    xbmc.log(f"REPO_OWNER: {REPO_OWNER}", xbmc.LOGINFO)
    xbmc.log(f"REPO_NAME: {REPO_NAME}", xbmc.LOGINFO)
    xbmc.log(f"GITHUB_TOKEN: {'*' * len(GITHUB_TOKEN)}", xbmc.LOGINFO)
    
    return True

def get_file_from_github(file_path):
    global REPO_OWNER, REPO_NAME, GITHUB_TOKEN

    if not check_github_settings():
        return None

    url = f'https://api.github.com/repos/{REPO_OWNER}/{REPO_NAME}/contents/{file_path}'
    
    headers = {
        'Authorization': f'token {GITHUB_TOKEN}',
        'Accept': 'application/vnd.github.v3.raw'
    }
    
    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.text
    except requests.exceptions.HTTPError as e:
        if e.response.status_code == 404:
            xbmc.log(f"File non trovato: {file_path}. URL: {url}", xbmc.LOGERROR)
        else:
            xbmc.log(f"Errore HTTP nel recupero del file {file_path}: {e}", xbmc.LOGERROR)
    except requests.exceptions.RequestException as e:
        xbmc.log(f"Errore nel recupero del file {file_path}: {str(e)}", xbmc.LOGERROR)
    return None

# Funzione per costruire un URL con parametri
def build_url(query):
    return f"{base_url}?{urlencode(query)}"

def save_credentials(username, password):
    # Salva le credenziali nel file settings.xml
    addon.setSetting('username', username)
    addon.setSetting('password', password)

def login():
    dialog = xbmcgui.Dialog()

    # Controlla se ci sono credenziali salvate
    saved_username = addon.getSetting('username')
    saved_password = addon.getSetting('password')

    if saved_username and saved_password:
        # Se esistono, usa le credenziali salvate
        if check_credentials(saved_username, saved_password):
            dialog.notification('Login', 'Login effettuato con successo', xbmcgui.NOTIFICATION_INFO, 3000)
            return True
        else:
            dialog.notification('Login', 'Credenziali scadute o non valide, reinserisci.', xbmcgui.NOTIFICATION_ERROR, 3000)

    # Se non ci sono credenziali salvate, chiedi all'utente di inserirle
    username = dialog.input('Inserisci Username')
    password = dialog.input('Inserisci Password', option=xbmcgui.ALPHANUM_HIDE_INPUT)
    
    if check_credentials(username, password):
        dialog.notification('Login', 'Login effettuato con successo', xbmcgui.NOTIFICATION_INFO, 3000)
        
        # Salva le credenziali nel file settings.xml
        save_credentials(username, password)

        return True
    else:
        dialog.notification('Login', 'Login fallito', xbmcgui.NOTIFICATION_ERROR, 3000)
        return False

# Funzione per mostrare il menu principale
def main_menu():
    if not login():
        return

    # Categorie principali
    categories = ["Live TV", "Film", "Serie TV"]
    for category in categories:
        list_item = xbmcgui.ListItem(label=category)
        url = build_url({'action': 'open_category', 'category': category})
        xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

    xbmcplugin.endOfDirectory(addon_handle)

# Funzione per aprire una categoria
def open_category(category):
    try:
        if category == "Film":
            subcategories = ["Azione", "Avventura", "Fantascienza", "Guerra", "Horror", "Dramma", "Crime", "Fantasy", "Mistery", "Thriller"]
            for subcategory in subcategories:
                list_item = xbmcgui.ListItem(label=subcategory)
                url = build_url({'action': 'initCategory', 'subcategory': subcategory, 'start':0,'end':20})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

        elif category == "Serie TV":
            subcategories = ["Dramma", "Commedia", "Fantascienza"]
            for subcategory in subcategories:
                list_item = xbmcgui.ListItem(label=subcategory)
                url = build_url({'action': 'load_series', 'subcategory': subcategory})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

        elif category == "Live TV":
            channels = load_m3u("file/file.m3u")
            if not channels:
                xbmcgui.Dialog().notification('Errore', 'Nessun canale trovato nel file M3U', xbmcgui.NOTIFICATION_ERROR, 3000)
                return
            
            for channel in channels:
                if 'staff_id' in channel and 'name' in channel:
                    list_item = xbmcgui.ListItem(label=channel['name'])
                    list_item.setInfo('video', {'title': channel['name']})
                    list_item.setProperty('IsPlayable', 'true')
                    if 'logo' in channel:
                        list_item.setArt({'thumb': channel['logo'], 'icon': channel['logo'], 'fanart': channel['logo']})
                    url = build_url({'action': 'play', 'content_type': 'live', 'id': channel['staff_id'], 'mpd_url': channel})
                    xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)

        xbmcplugin.endOfDirectory(addon_handle)

    except Exception as e:
        xbmcgui.Dialog().notification('Errore', f"Errore caricamento categoria: {str(e)}", xbmcgui.NOTIFICATION_ERROR, 5000)
        xbmc.log(f"Exception in open_category: {str(e)}", xbmc.LOGERROR)

def get_info(title, content_type):
    tmdb_api_key = "ded5b7afc33c6be7e0eaefce452217b1"
    r = requests.get(f"https://api.themoviedb.org/3/search/{content_type}?api_key={tmdb_api_key}&language=it-IT&query={requests.utils.quote(title)}")
    if r  and r.json():
        return r.json()

def initCategory(subcategory):
    if subcategory == "Azione":
        m3u_url = 'file/Film/azione1.m3u'
        m3u_url = 'file/Film/azione2.m3u'
        m3u_url = 'file/Film/azione3.m3u'
        m3u_url = 'file/Film/azione4.m3u'
        m3u_url = 'file/Film/azione5.m3u'
        m3u_url = 'file/Film/azione6.m3u'
        m3u_url = 'file/Film/azione7.m3u'
        m3u_url = 'file/Film/azione8.m3u'
    elif subcategory == "Avventura":
        m3u_url = 'file/Film/avventura1.m3u'
        m3u_url = 'file/Film/avventura2.m3u'
    elif subcategory == "Avventura":
        m3u_url = 'file/Film/avventura.m3u'
    elif subcategory == "Fantascienza":
        m3u_url = 'file/Film/fantascienza.m3u'
    elif subcategory == "Guerra":
        m3u_url = 'file/Film/guerra.m3u'
    elif subcategory == "Horror":
        m3u_url = 'file/Film/horror1.m3u'
        m3u_url = 'file/Film/horror2.m3u'
        m3u_url = 'file/Film/horror3.m3u'
        m3u_url = 'file/Film/horror4.m3u'
    elif subcategory == "Dramma":
        m3u_url = 'file/Film/dramma1.m3u'
        m3u_url = 'file/Film/dramma2.m3u'
        m3u_url = 'file/Film/dramma3.m3u'
        m3u_url = 'file/Film/dramma4.m3u'
        m3u_url = 'file/Film/dramma5.m3u'
        m3u_url = 'file/Film/dramma6.m3u'
        m3u_url = 'file/Film/dramma7.m3u'
        m3u_url = 'file/Film/dramma8.m3u'
    elif subcategory == "Crime":
        m3u_url = 'file/Film/crime1.m3u'
        m3u_url = 'file/Film/crime2.m3u'
        m3u_url = 'file/Film/crime3.m3u'
        m3u_url = 'file/Film/crime4.m3u'
        m3u_url = 'file/Film/crime5.m3u'
        m3u_url = 'file/Film/crime6.m3u'
    elif subcategory == "Fantasy":
        m3u_url = 'file/Film/fantasy1.m3u'
        m3u_url = 'file/Film/fantasy2.m3u'
        m3u_url = 'file/Film/fantasy3.m3u'
        m3u_url = 'file/Film/fantasy4.m3u'
        m3u_url = 'file/Film/fantasy5.m3u'
        m3u_url = 'file/Film/fantasy6.m3u'
    elif subcategory == "Mistery":
        m3u_url = 'file/Film/mistery1.m3u'
        m3u_url = 'file/Film/mistery2.m3u'
        m3u_url = 'file/Film/mistery3.m3u'
        m3u_url = 'file/Film/mistery4.m3u'
    elif subcategory == "Thriller":
        m3u_url = 'file/Film/thriller1.m3u'
        m3u_url = 'file/Film/thriller2.m3u'
        m3u_url = 'file/Film/thriller3.m3u'
        m3u_url = 'file/Film/thriller4.m3u'
        m3u_url = 'file/Film/thriller5.m3u'
        m3u_url = 'file/Film/thriller6.m3u'

    load_movies(load_m3u(m3u_url), 0, 10)

def load_movies(channels, start, end):
    if not isinstance(channels, list):
        xbmc.log(f"Error: channels is not a list: {channels}", level=xbmc.LOGERROR)
        return

    xbmc.log(f"Channel: {channels[start:end]}", level=xbmc.LOGERROR)

    for movie in channels[start:end]:
        if not isinstance(movie, dict):
            continue

        title = movie.get('name', 'Titolo non disponibile')
        info_film = get_info(title, "movie")
        
        if info_film and len(info_film["results"]) > 0:
            movie_info = info_film["results"][0]
            
            poster_path = movie_info.get('poster_path', '')
            backdrop_path = movie_info.get('backdrop_path', '')
            overview = movie_info.get('overview', 'Descrizione non disponibile')

            list_item = xbmcgui.ListItem(label=title)
            list_item.setInfo('video', {
                'title': title,
                'plot': overview
            })

            list_item.setArt({
                'thumb': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{poster_path}" if poster_path else '',
                'fanart': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{backdrop_path}" if backdrop_path else ''
            })
            
            url = build_url({'action': 'play', 'content_type': 'movie', 'title': title, 'mpd_url': movie})
            xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)
    
    if end < len(channels):
        _start = end
        _end = _start + 10 if _start + 10 < len(channels) else len(channels)
        
        next_page_url = build_url({
            'action': 'next_page',
            'start': _start,
            'end': _end,
            'channels': json.dumps(channels)  # Converti in JSON per passare i parametri
        })
        next_page_item = xbmcgui.ListItem(label='Pagina successiva >>')
        xbmcplugin.addDirectoryItem(handle=addon_handle, url=next_page_url, listitem=next_page_item, isFolder=True)

    xbmcplugin.endOfDirectory(addon_handle)

# Funzione per caricare il file M3U da un URL e restituire i dati
def load_m3u(url):
    channels = []
    current_channel = {}  # Aggiunta questa variabile per tenere traccia del canale corrente
    content = get_file_from_github(url)
    
    if content is None:
        xbmc.log(f"Impossibile caricare il file da GitHub: {url}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', f'Impossibile caricare il file: {url}', xbmcgui.NOTIFICATION_ERROR, 5000)
        return []
    
    # Divide il contenuto in righe
    lines = content.splitlines()

    # Itera su ogni riga del file M3U
    for line in lines:
        line = line.strip()
        if line.startswith('#EXTINF'):
            info = line.split(',')
            channel_info = info[0].split(' ')
            current_channel['name'] = info[1].strip()
            for attr in channel_info[1:]:
                if attr.startswith('staff-id='):
                    current_channel['staff_id'] = attr.split('=')[1].strip('"')
                elif attr.startswith('tvg-logo='):
                    current_channel['logo'] = attr.split('=')[1].strip('"')
                elif attr.startswith('group-title='):
                    current_channel['group_title'] = attr.split('=')[1].strip('"')
        elif line.startswith('#KODIPROP'):
            if 'license_key=' in line:
                current_channel['license_key'] = line.split('=')[1].strip()
        elif line.startswith('http'):
            current_channel['mpd_url'] = line
            channels.append(current_channel)
            current_channel = {}

    return channels

def play_content(info):
    try:
        info = info.replace("'", '"')
        channel = json.loads(info)
        if channel and 'mpd_url' in channel:
            user_agent = 'User-Agent=Mozilla/5.0 (Web0S; Linux/SmartTV) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.128 Safari/537.36 WebAppManager'
            xbmc.log(f"Channel data: {channel}", level=xbmc.LOGINFO)
            play_url = channel['mpd_url']

            list_item = xbmcgui.ListItem(path=play_url)

            # Imposta le chiavi ClearKey
            if 'license_key' in channel:
                hex_key = channel['license_key']
                list_item.setProperty('inputstream.adaptive.drm_legacy', 'org.w3.clearkey|' + hex_key)
            
            # Aggiungi stream_params solo per MPD specifici
            if "mpd" in play_url:
                list_item.setProperty('inputstream', 'inputstream.adaptive')
                list_item.setProperty('inputstream.adaptive.manifest_type', 'mpd')
                list_item.setProperty('inputstream.adaptive.stream_params', '?p=web')
                list_item.setProperty('inputstream.adaptive.stream_headers', user_agent)
                list_item.setProperty('inputstream.adaptive.manifest_headers', user_agent)
                list_item.setProperty('inputstream.adaptive.max_resolution', 'max')
            else:
                play_url = play_url +'|User-Agent='+user_agent
                xbmc.executebuiltin("PlayMedia(" + play_url + ")")
            # Risolvi l'URL
            xbmcplugin.setResolvedUrl(addon_handle, True, listitem=list_item)
        else:
            xbmc.log("Canale non trovato o manca l'URL MPD", level=xbmc.LOGERROR)
    except Exception as e:
        xbmc.log(f"Errore generale in play_content: {str(e)}", level=xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', 'Errore durante la riproduzione', xbmcgui.NOTIFICATION_ERROR, 5000)

# Router per gestire le azioni
def router(paramstring):
    params = dict(parse_qsl(paramstring[1:]))
    if params:
        if params['action'] == 'open_category':
            open_category(params['category'])
        elif params['action'] == 'initCategory':
            initCategory(params['subcategory'])
        elif params['action'] == 'next_page':
            channels = json.loads(params['channels'])
            load_movies(channels, int(params['start']), int(params['end']))
        elif params['action'] == 'play':
            play_content(params['mpd_url'])
    else:
        main_menu()

# Avvio dell'addon
if __name__ == '__main__':
    router(sys.argv[2])